//
//  charts.js
//  Library
//
//  Created by Jonathon Mah on 2013-03-16.
//  Copyright (c) 2013 Delicious Monster Software. All rights reserved.
//


/* construct name so genstrings doesn't barf */
window["jsLocalized" + "StringFromTable"] = function(key, table, comment) {
    return window.dataProvider.localizedChartsString(key);
};

window.iCap = function(s) {
    return window.dataProvider.iCapitalizeString(s);
};

window.copyMediaAttributes = function(keys) {
    var jsonKeys = JSON.stringify(keys);
    var jsonAttributes = window.dataProvider.mediaAttributeDictionariesWithKeys(jsonKeys);
    return JSON.parse(jsonAttributes);
};

window.mediaAttributeSumForKeys = function(keys, predicateString) {
    var jsonKeys = JSON.stringify(keys);
    return window.dataProvider.mediaAttributeSumForKeysWithPredicate(jsonKeys, (predicateString || ""));
};

window.mediaAttributeProportions = function(key) {
    var mediaCount = window.dataProvider.mediaCount();
    var valuesAndCounts = window.dataProvider.mediaAttributeCountsForKey(key);

    var countProportions = [];
    for (var i = 0; i < valuesAndCounts.length; i++) {
        var valueAndCount = valuesAndCounts[i];
        countProportions.push([valueAndCount[0], (valueAndCount[1] / mediaCount)]);
    }
    return countProportions;
};

window.formatCurrency = function(v) {
    return dataProvider.formatValueInLocalCurrency(v);
};

window.formatCurrencyShort = function(v) {
    /// TODO: localize better
    var s = dataProvider.formatValueInLocalCurrency(v);
    var lastThreeChars = s.substr(s.length - 3);
    if (lastThreeChars == ".00" || lastThreeChars == ",00")
        return s.substring(0, s.length - lastThreeChars.length);
    else
        return s;
};


if (!window.dataProvider) {
    // Stubs for debugging in browser
    window.dataProvider = {
        _filterMediaType: "",
        unfilteredMediaTypeCounts: function() { return '[{"placeholderCoverURL":"file:\/\/localhost\/Users\/jonathon\/Library\/Developer\/Xcode\/DerivedData\/UberLibrary-djzgxxrdrlqezxgaansiwhdraxmn\/Build\/Products\/Debug%20(Mac%20App%20Store)\/Delicious%20Library%203.app\/Contents\/Resources\/Book_placeholderCover.tiff","mediaType":"Book","count":6,"localizedPluralName":"books"},{"placeholderCoverURL":"file:\/\/localhost\/Users\/jonathon\/Library\/Developer\/Xcode\/DerivedData\/UberLibrary-djzgxxrdrlqezxgaansiwhdraxmn\/Build\/Products\/Debug%20(Mac%20App%20Store)\/Delicious%20Library%203.app\/Contents\/Resources\/Movie_placeholderCover.tiff","mediaType":"Movie","count":6,"localizedPluralName":"movies"},{"placeholderCoverURL":"file:\/\/localhost\/Users\/jonathon\/Library\/Developer\/Xcode\/DerivedData\/UberLibrary-djzgxxrdrlqezxgaansiwhdraxmn\/Build\/Products\/Debug%20(Mac%20App%20Store)\/Delicious%20Library%203.app\/Contents\/Resources\/VideoGame_placeholderCover.tiff","mediaType":"VideoGame","count":4,"localizedPluralName":"videogames"},{"placeholderCoverURL":"file:\/\/localhost\/Users\/jonathon\/Library\/Developer\/Xcode\/DerivedData\/UberLibrary-djzgxxrdrlqezxgaansiwhdraxmn\/Build\/Products\/Debug%20(Mac%20App%20Store)\/Delicious%20Library%203.app\/Contents\/Resources\/Tool_placeholderCover.tiff","mediaType":"Tool","count":3,"localizedPluralName":"tools"}]'; },
        libraryUUID: function() { return "879A4507-432C-4A01-811E-E2F8EC6FBB2E"; },
        mediaCount: function() { return 30; },

        log: function(s) { console.log(s); },
        iCapitalizeString: function(s) { return s.charAt(0).toUpperCase() + s.substr(1); },
        localizedChartsString: function(key) { return key; },
        localizedNameForMediaType: function(type) { return type; },
        localizedPluralNameForMediaType: function(type) { return (type + "s"); },
        localizedNameForMediaKey: function(key, count) { return /^[a-z]+/.exec(key)[0] + (count != 1 ? "s" : ""); },
        formatValueInLocalCurrency: function(val) { return sprintf("$%0.2f", val); },
        formatDurationInMinutes: function(val) { return sprintf("%.0f min", val); },
        mediaAttributeCountsForKey: function(key) { return [["foo",12],["bar",15],["baz",3]]; },
        mediaAttributeSumForKeysWithPredicate: function(keys, pred) { return pred ? 22 : 42; },
        mediaAttributeDictionariesWithKeys: function(keys) { return JSON.stringify([]); },
        filterMediaType: function() { return this._filterMediaType; },
        setFilterMediaType: function(type) { this._filterMediaType = type; setTimeout(window.updateCharts, 0); }
    };
}



(function(window) {
    var _updatable = [];

    window.onUpdateCharts = function(fn) {
        _updatable.push(fn);
    };

    window.updateCharts = function() {
        // This function is called from LIShelfSourceViewController+ChartsWebView when data changes
        for (var i = 0; i < _updatable.length; i++) {
            Math.seedrandom(window.dataProvider.libraryUUID() + i);
            try {
                _updatable[i]();
            } catch (e) {
                dataProvider.log("" + e);
            }
        }

        if (window.masonry)
            window.masonry.reload();
    };
})(window);


var chartGridSize = 60;

function addChart(chart, widthInGridUnits, heightInGridUnits, title) {
    var widthPadding = 24; // Unfortunately must do padding here, or Masonry won't lay out as we like
    var titleHeight = 38;
    var widthInPixels = widthInGridUnits * chartGridSize - widthPadding;
    var heightInPixels = heightInGridUnits * 75;

    // Note: Title is NOT included in width/height because it's easier at this point. They should all be the same height so it shouldn't matter anyway.
    // TODO: Can we bring title inside height please
    var chartDiv = d3.select("#chart-container")
        .append("div")
        .classed("chart", true)
        .style("width", widthInPixels + "px")
        .style("height", heightInPixels + "px");

    var titleDiv =
        chartDiv
        .append("div")
        .classed("title", true);

    function updateTitle(newTitle) {
        titleDiv
            .text(newTitle);

        chartDiv.style("width", widthInPixels + "px"); // reset back to original to re-measure scrollWidth
        chartDiv.style("width", Math.max(titleDiv.node().scrollWidth, widthInPixels) + "px");
    }

    if (typeof title === "function") {
        onUpdateCharts(function() { updateTitle(title()); });
    } else {
        updateTitle(title);
    }

    var svg =
        chartDiv
        .append("svg")
        .attr("width", widthInPixels)
        .attr("height", heightInPixels - titleHeight);

    var chart = new chart(chartDiv, svg, widthInPixels, heightInPixels - titleHeight);
    onUpdateCharts(function() { chart.update(); });
    return chart;
}
